import Flutter
import UIKit
import AppBoxoSDK

public class SwiftAppboxoSdkPlugin: NSObject, FlutterPlugin, FlutterStreamHandler {
    
    private var events: FlutterEventSink? = nil
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        let instance = SwiftAppboxoSdkPlugin()
        
        let channel = FlutterMethodChannel(name: "appboxo_sdk", binaryMessenger: registrar.messenger())
        registrar.addMethodCallDelegate(instance, channel: channel)
        
        let eventChannel = FlutterEventChannel(name: "appboxo_sdk_custom_events", binaryMessenger: registrar.messenger())
        eventChannel.setStreamHandler(instance)
    }
    
    
    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        
        switch call.method {
        case "setConfig":
            guard let arguments = call.arguments as? [String : Any] else { return }
            let clientId = arguments["client_id"] as? String ?? ""
            AppBoxo.shared.setConfig(config: Config(clientId: clientId))
        case "openMiniApp":
            guard let arguments = call.arguments as? [String : Any] else { return }
            let appId = arguments["app_id"] as? String ?? ""
            let payload = arguments["payload"] as? String ?? ""
            let miniApp = AppBoxo.shared.getMiniApp(appId: appId, authPayload: payload)
            miniApp.open(viewController: UIApplication.shared.delegate!.window!!.rootViewController!)
            break
        case "openMiniAppWithCustomEvents":
            guard let arguments = call.arguments as? [String : Any] else { return }
            let appId = arguments["app_id"] as? String ?? ""
            let payload = arguments["payload"] as? String ?? ""
            let miniApp = AppBoxo.shared.getMiniApp(appId: appId, authPayload: payload)
            miniApp.delegate = self
            miniApp.open(viewController: UIApplication.shared.delegate!.window!!.rootViewController!)
            break
        case "sendEvent":
            guard let arguments = call.arguments as? [String : Any] else { return }
            let map = arguments["custom_event"] as? [String: Any] ?? [:]
            let appId = map["appId"] as? String ?? ""
            
            var params = [
                "request_id" : map["requestId"] ?? 0,
                "type" : map["type"] ?? "",
                "payload" : map["payload"] ?? [String : Any]()
                ] as [String: Any]
            
            if let errorType = map["errorType"] as? String {
                params["error_type"] = errorType
            }
            
            AppBoxo.shared.getMiniApp(appId: appId)?.sendEvent(params: params)
            break
        default:
            break
        }
    }
    
    public func onListen(withArguments arguments: Any?, eventSink events: @escaping FlutterEventSink) -> FlutterError? {
        self.events = events
        return nil
    }
    
    public func onCancel(withArguments arguments: Any?) -> FlutterError? {
        self.events = nil
        return nil
    }
}

extension SwiftAppboxoSdkPlugin : MiniAppDelegate {
    public func didReceiveCustomEvent(miniApp: MiniApp, params: [String : Any]) {
        print("params: \(params)")
        
        var dict = [
            "appId" : miniApp.appId,
            "requestId" : "\(params["request_id"] ?? 0)",
            "type" : params["type"] ?? "",
            "payload" : params["payload"] ?? [String : Any]()
            ] as [String: Any]
        
        if let errorType = params["errorType"] as? String {
            dict["error_type"] = errorType
        }
        
        self.events?(dict)
    }
}
