import 'package:flutter/services.dart';

class AppboxoSdk {
  static const MethodChannel _channel = const MethodChannel('appboxo_sdk');
  static const EventChannel _customEventChannel = const EventChannel('appboxo_sdk_custom_events');

  static void setConfig(String clientId) {
    _channel.invokeMethod("setConfig", {'client_id': clientId});
  }

  static void openMiniApp(String appId, String payload) {
    _channel.invokeMethod("openMiniApp", {'app_id': appId, 'payload': payload});
  }

  static void openMiniAppWithCustomEvents(String appId, String payload) {
    _channel.invokeMethod("openMiniAppWithCustomEvents", {'app_id': appId, 'payload': payload});
  }

  static Stream<CustomEvent> customEvents() {
    return _customEventChannel.receiveBroadcastStream().map((dynamic event) => CustomEvent.fromJson(event));
  }

  static void sendEvent(CustomEvent customEvent) {
    _channel.invokeMethod("sendEvent", {'custom_event': customEvent.toJson()});
  }
}

class CustomEvent {
  String appId;
  String requestId;
  String type;
  String errorType;
  Map payload;

  CustomEvent(this.appId, this.requestId, this.type, this.errorType, this.payload);

  CustomEvent.fromJson(Map<dynamic, dynamic> json)
      : appId = json['appId'],
        requestId = json['requestId'],
        type = json['type'],
        errorType = json['errorType'],
        payload = json['payload'];

  Map<String, dynamic> toJson() =>
      {'appId': appId, 'requestId': requestId, 'type': type, 'errorType': errorType, 'payload': payload};
}
